// Implementation code for signal/slot classes
// It has to be split from the main header to avoid
// "invalid use of undefined class ..." errors.
namespace sigslot {

	has_slots::has_slots()
	{}

	has_slots::~has_slots()
	{
		disconnect_all();
	}

	has_slots::sender_set & has_slots::signalers()
	{
		return this->m_senders;
	}
	
	const has_slots::sender_set & has_slots::signalers() const
	{
		return this->m_senders;
	}


	has_slots::has_slots( const has_slots & rhs )
	{
		this->copy_signalers(rhs);
	} 

	has_slots & has_slots::operator=(const has_slots& rhs)
	{
		this->disconnect_all();
		this->copy_signalers(rhs);
		return *this;
	}

	void has_slots::add_signaler(signal_base* sender)
	{
		this->signalers().insert(sender);
	}

	void has_slots::remove_signaler(signal_base* sender)
	{
		this->signalers().erase(sender);
	}

	void has_slots::copy_signalers( const has_slots & rhs )
	{
		if( &rhs == this ) return;
		const_iterator it = rhs.signalers().begin();
		const_iterator itEnd = rhs.signalers().end();
		for( ; it != itEnd; ++it )
		{
			(*it)->slot_duplicate(&rhs, this);
			this->signalers().insert(*it);
		}
	}

	void has_slots::disconnect_all()
	{
		iterator it = this->signalers().begin();
		iterator et = this->signalers().end();
		for( ; et != it; ++it )
		{
			(*it)->disconnect_slot( this );
		}
		this->signalers().clear();
	}



} // namespace sigslot
